/******************************************************************************

 @file  throughput_peripheral_menu.c

 @brief This file contains menu application for throughput_peripheral.

 Group: WCS BTS
 Target Device: cc13x2_2x2

 ******************************************************************************

 Copyright (c) 2016-2020, Texas Instruments Incorporated
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 *  Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 *  Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.

 *  Neither the name of Texas Instruments Incorporated nor the names of
 its contributors may be used to endorse or promote products derived
 from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 ******************************************************************************


 *****************************************************************************/

#include <bcomdef.h>
#include <ti/display/Display.h>

#include <menu/two_btn_menu.h>
#include <throughput_peripheral.h>
#include <throughput_peripheral_menu.h>
#include "ti_ble_config.h"

/* Main Menu Object */
tbmMenuObj_t tpMenuMain;
tbmMenuObj_t tpMenuSelectConn;
tbmMenuObj_t tpMenuPerConn;
tbmMenuObj_t tpMenuConnPhy;

/*
 * Menu Lists Initializations
 */

void Throughput_buildMenu(void)
{
// Menu: Main
// upper: none
MENU_OBJ(tpMenuMain, NULL, 1, NULL)
MENU_ITEM_SUBMENU(tpMenuMain, 0, &tpMenuSelectConn)
MENU_OBJ_END

// Menu: SelectDev
// upper: Main
// NOTE: The number of items in this menu object shall be
//       equal to or greater than MAX_NUM_BLE_CONNS
MENU_OBJ(tpMenuSelectConn, "Work with", 1, &tpMenuMain)
MENU_ITEM_ACTION(tpMenuSelectConn, 0, NULL, ThroughputPeripheral_doSelectConn)
MENU_OBJ_END

// Menu: PerConnection
// upper: SelectDevice
MENU_OBJ(tpMenuPerConn, NULL, 2, &tpMenuSelectConn)
MENU_ITEM_SUBMENU(tpMenuPerConn, 0, &tpMenuConnPhy)
MENU_ITEM_ACTION(tpMenuPerConn, 1, "Toggle Throughput",   ThroughputPeripheral_doToggleThroughput)
MENU_OBJ_END

// Menu: ConnPhy
// upper: Select Device
MENU_OBJ(tpMenuConnPhy, "Set Conn PHY Preference", 4, &tpMenuPerConn)
MENU_ITEM_ACTION(tpMenuConnPhy, 0, "1 Mbps",              ThroughputPeripheral_doSetConnPhy)
MENU_ITEM_ACTION(tpMenuConnPhy, 1, "2 Mbps",              ThroughputPeripheral_doSetConnPhy)
MENU_ITEM_ACTION(tpMenuConnPhy, 2, "Coded:S2",            ThroughputPeripheral_doSetConnPhy)
MENU_ITEM_ACTION(tpMenuConnPhy, 3, "Coded:S8",            ThroughputPeripheral_doSetConnPhy)
MENU_OBJ_END
}
